/**
 *******************************************************************************
 * @file        A33G53x_frt.h
 * @author      ABOV R&D Division
 * @brief       Contains all macro definitions and function prototypes
 *              support for FRT driver on A33G53x
 *
 * Copyright 2023 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

#include <stdint.h>
#include "A33G53x.h"


//==========================================================================
// 	FRTPRD
//		
//				@ address = 0x4000_0500
//
//==========================================================================





//==========================================================================
// 	FRTCNT
//		
//				@ address = 0x4000_0504
//
//==========================================================================





//==========================================================================
// 	FRTCON
//		
//				@ address = 0x4000_0508
//
//==========================================================================
#define FRTCON_FMF							(0x0001<<9)
#define FRTCON_FOF							(0x0001<<8)

#define FRTCON_FEC							(0x0001<<7)
#define FRTCON_FEC_PCLK						(0x0000<<7)
#define FRTCON_FEC_PMUPCSR					(0x0001<<7)

#define FRTCON_FPRS_FRTCLKIN_DIV_BY_1		(0x0000<<4)
#define FRTCON_FPRS_FRTCLKIN_DIV_BY_4		(0x0001<<4)
#define FRTCON_FPRS_FRTCLKIN_DIV_BY_8		(0x0002<<4)
#define FRTCON_FPRS_FRTCLKIN_DIV_BY_16		(0x0003<<4)
#define FRTCON_FPRS_FRTCLKIN_DIV_BY_32		(0x0004<<4)
#define FRTCON_FPRS_FRTCLKIN_DIV_BY_64		(0x0005<<4)
#define FRTCON_FPRS_FRTCLKIN_DIV_BY_128		(0x0006<<4)
#define FRTCON_FPRS_FRTCLKIN_DIV_BY_256		(0x0007<<4)
#define FRTCON_FPRS_MASK					(0x0007<<4)

#define FRTCON_FMIE							(0x0001<<3)
#define FRTCON_FOIE							(0x0001<<2)

#define FRTCON_FAC							(0x0001<<1)
#define FRTCON_FAC_FREERUN					(0x0000<<1)
#define FRTCON_FAC_PERIODIC					(0x0001<<1)



#define FRTCON_FEN							(0x0001<<0)








//==========================================================================
// 
//		F U N C T I O N    D E C L A R A T I O N S 
//
//==========================================================================
void FRT_ConfigureInterrupt (FRT_Type  * const frt, uint32_t intr_mask, uint32_t enable); 






