/**
 *******************************************************************************
 * @file        A33G53x_frt.c
 * @author      ABOV R&D Division
 * @brief       Contains all macro definitions and function prototypes
 *              support for FRT driver on A33G53x
 *
 * Copyright 2023 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/


#include <stdint.h>
#include "A33G53x.h"
#include "A33G53x_frt.h"
#include "A33G53x_nvic.h"


/**
*********************************************************************************************************
* @ Name : FRT_ConfigureInterrupt 
*
* @ Parameters
*		- frt : FRT
*		- intr_mask : FRTCON_FMIE, FRTCON_FOIE
*		- enable : INTR_ENABLE, INTR_DISABLE
*
*
*********************************************************************************************************
*/
void FRT_ConfigureInterrupt (FRT_Type * const frt, uint32_t intr_mask, uint32_t enable)
{
	uint32_t			reg_val; 
	


	//------------------------------------------------------------------------------
	// disable interrupt 
	//
	//				FRTCON			@ address = 0x4000_0508
	//
	//------------------------------------------------------------------------------
	reg_val = frt->CON;
	reg_val &= ~(FRTCON_FMIE|FRTCON_FOIE); 
	frt->CON = reg_val;
	

	//------------------------------------------------------------------------------
	// clear interrupt flag 
	//
	//				FRTCON			@ address = 0x4000_0508
	//
	//------------------------------------------------------------------------------
	reg_val &= ~(FRTCON_FMF|FRTCON_FOF); 
	frt->CON = reg_val;

	

	//------------------------------------------------------------------------------
	// enable interrupt 
	//
	//				FRTCON			@ address = 0x4000_0508
	//
	//------------------------------------------------------------------------------
	if (enable == INTR_ENABLE)
	{
		reg_val |= intr_mask; 
		frt->CON = reg_val;
	}

}


